<?php

/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2017 All rights reserved
 * @package DZF
 */

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
class DZF_Sms_Transport_Sipgate extends DZF_Sms_Transport_Abstract
{
    const API_ENDPOINT = 'https://api.sipgate.net/RPC2';

    const TOS_FAX   = 'fax';
    const TOS_TEXT  = 'text';
    const TOS_VIDEO = 'video';
    const TOS_VOICE = 'voice';

    const MESSAGE_MAX_LENGTH = 160;

    const METHOD_LIST_METHODS       = 'system.listMethods';
    const METHOD_SERVER_INFO        = 'system.serverInfo';
    const METHOD_BALANCE_GET        = 'samurai.BalanceGet';
    const METHOD_SESSION_CLOSE      = 'samurai.SessionClose';
    const METHOD_SESSION_INITIATE   = 'samurai.SessionInitiate';
    const METHOD_TOS_LIST_GET       = 'samurai.TosListGet';

    /**
     * @var string
     */
    protected $_username;

    /**
     * @var string
     */
    protected $_password;

    /**
     * @var string
     */
    protected $_senderPhoneNumber;

    /**
     * @var array
     */
    protected $_response = array();

    /**
     * @param   string          $username
     * @param   string          $password
     * @param   string|null     $senderPhoneNumber      (optional, default - null)
     */
    public function __construct($username, $password, $senderPhoneNumber = null)
    {
        $this->setUsername($username);
        $this->setPassword($password);

        if ($senderPhoneNumber)
        {
            $this->setSenderPhoneNumber($senderPhoneNumber);
        }
    }

    /**
     * @param   string          $username
     * @return  DZF_Sms_Transport_Sipgate
     */
    public function setUsername($username)
    {
        $this->_username = $username;
        return $this;
    }

    /**
     * @param   string          $password
     * @return  DZF_Sms_Transport_Sipgate
     */
    public function setPassword($password)
    {
        $this->_password = $password;
        return $this;
    }

    /**
     * @param   string          $senderPhoneNumber
     * @return  DZF_Sms_Transport_Sipgate
     */
    public function setSenderPhoneNumber($senderPhoneNumber)
    {
        $this->_senderPhoneNumber = $senderPhoneNumber;
        return $this;
    }

    /**
     * @return  array
     */
    public function getAllResponse()
    {
        return $this->_response;
    }

    /**
     * @return  array|null
     */
    public function getResponseData()
    {
        return isset($this->_response['response']) ? $this->_response['response'] : null;
    }

    /**
     * @return  int|null
     */
    public function getResponseStatusCode()
    {
        return isset($this->_response['status_code']) ? $this->_response['status_code'] : null;
    }

    /**
     * @return  string|null
     */
    public function getResponseStatusString()
    {
        return isset($this->_response['status_string']) ? $this->_response['status_string'] : null;
    }

    /**
     * @return  string|null
     */
    public function getResponseSessionId()
    {
        return isset($this->_response['session_id']) ? $this->_response['session_id'] : null;
    }

    /**
     * @return  string|null
     */
    public function getLastRawRequest()
    {
        return isset($this->_response['raw_request']) ? $this->_response['raw_request'] : null;
    }

    /**
     * @return  string|null
     */
    public function getLastRawResponse()
    {
        return isset($this->_response['raw_response']) ? $this->_response['raw_response'] : null;
    }

    /**
     * Try to send SMS to the specified recipient.
     * Max length - 160 characters.
     *
     * @param   string          $phoneNumber
     * @param   string          $content
     * @param   array           $params
     * @return  boolean
     */
    public function sendSms($phoneNumber, $content, $params = array())
    {
        $this->_response = array();

        $client = $this->getXmlRpcClient();

        try
        {
            $callParams = array(
                'RemoteUri' => 'sip:' . $phoneNumber . '@sipgate.de',
                'TOS'       => self::TOS_TEXT,
                'Content'   => substr($content, 0, self::MESSAGE_MAX_LENGTH - 1)
            );

            if ($this->_senderPhoneNumber)
            {
                $callParams['LocalUri'] = 'sip:' . $this->_senderPhoneNumber . '@sipgate.de';
            }

            $result = $client->call('samurai.SessionInitiate', array($callParams));

            $this->_response = array(
                'raw_request'   => $client->getHttpClient()->getLastRequest(),
                'raw_response'  => $client->getHttpClient()->getLastResponse()->asString(),
                'response'      => $result,
                'status_code'   => isset($result['StatusCode']) ? $result['StatusCode'] : null,
                'status_string' => isset($result['StatusString']) ? $result['StatusString'] : null,
                'session_id'    => isset($result['SessionID']) ? $result['SessionID'] : null
            );

            return true;
        }
        catch (Exception $e)
        {
            $this->_response = array(
                'raw_request'   => $client->getHttpClient()->getLastRequest(),
                'raw_response'  => $client->getHttpClient()->getLastResponse()->asString(),
                'status_code'   => $e->getCode(),
                'status_string' => $e->getMessage()
            );

            return false;
        }
    }

    /**
     * @var Zend_XmlRpc_Client
     */
    protected static $_client;

    /**
     * @return  Zend_XmlRpc_Client
     */
    public function getXmlRpcClient()
    {
        if (!isset(self::$_client))
        {
            self::$_client = new Zend_XmlRpc_Client(self::API_ENDPOINT);
        }

        self::$_client->getHttpClient()->resetParameters(true);
        self::$_client->getHttpClient()->setAuth($this->_username, $this->_password);
        self::$_client->getHttpClient()->setHeaders('Accept-Encoding', 'identity');
        //self::$_client->getHttpClient()->setHeaders('Transfer-Encoding', 'identity');

        return self::$_client;
    }

    /**
     * @param   Zend_XmlRpc_Client      $client
     */
    public static function setXmlRpcClient($client)
    {
        self::$_client = $client;
    }
}
